import pydantic
from typing import Union


class ProxyMetadataModel(pydantic.BaseModel):
    status: str
    country: str
    countryCode: str
    region: str
    regionName: str
    city: str
    zip: str
    lat: float
    lon: float
    timezone: str
    isp: str
    org: str
    as_: str = pydantic.Field(..., alias="as")
    query: str
    response_time: Union[float, None] = None
    continent: Union[str, None] = None
    continentCode: Union[str, None] = None
    district: Union[str, None] = None
    offset: Union[float, None] = None
    currency: Union[str, None] = None
    asname: Union[str, None] = None
    reverse: Union[str, None] = None
    mobile: Union[bool, None] = None
    proxy: Union[bool, None] = None
    hosting: Union[bool, None] = None



from pyfreeproxies.proxies import FreeProxies

from importlib import metadata

try:
    __version__ = metadata.version("pyfreeproxies")
except metadata.PackageNotFoundError:
    __version__ = "0.0.0"



from pyfreeproxies.models import ProxyMetadataModel
import pyfreeproxies.utils as proxies_util
from datetime import datetime, timezone, timedelta
import typing


class FreeProxies:
    """Contains free `http`, `socks4` and `socks5` proxies."""

    def __init__(self):
        """Constructor"""
        self.last_proxies_updated_time_str_in_utc: str = proxies_util.fetch(
            url=proxies_util.url_map["last_proxies_updated_time"]
        ).get("utc")
        self.available_proxies_category: list[str] = [
            "http",
            "socks4",
            "socks5",
            "random",
        ]

    @property
    def proxies_update_available(self) -> bool:
        """Check if there's an update to proxies"""
        last_update_time_in_utc = datetime.fromisoformat(
            self.last_proxies_updated_time_str_in_utc
        ).replace(tzinfo=timezone.utc)
        current_time_in_utc = datetime.now(timezone.utc)
        return current_time_in_utc > (
            last_update_time_in_utc
            + timedelta(seconds=proxies_util.proxies_update_frequency_in_seconds)
        )

    def get_proxies(
        self, proxy_type: typing.Literal["http", "socks4", "socks5", "random"]
    ) -> list[str]:
        """Get proxies"""
        assert (
            proxy_type in self.available_proxies_category
        ), f"Invalid proxy category {proxy_type}, should be one of {', '.join(self.available_proxies_type)}."
        return proxies_util.fetch(proxies_util.url_map[f"{proxy_type}_proxies"]).get(
            "proxies"
        )

    def get_http_proxies(self) -> list[str]:
        """Http proxies"""
        return self.get_proxies("http")

    def get_socks4_proxies(self) -> list[str]:
        """Socks4 proxies"""
        return self.get_proxies("socks4")

    def get_socks5_proxies(self) -> list[str]:
        """Socks5 proxies"""
        return self.get_proxies("socks5")

    def get_random_proxies(self) -> list[str]:
        """Random proxies"""
        return self.get_proxies("random")

    def get_combined_proxies(self) -> dict[str, list[str]]:
        """http, socks4 and socks5 proxies"""
        return proxies_util.fetch(proxies_util.url_map["combined_proxies"])

    def get_proxies_metadata(
        self, filters: dict[str, typing.Any] = {}
    ) -> dict[str, ProxyMetadataModel]:
        """Proxies with their info

        Args:
            filters (dict[str, typing.Any]): Proxy metadata key and it's corresponding value. Defaults to `{}`.
            - Case type of filter is `int|float`, comparator will be `<=` else `==`

        Returns:
            dict[str, ProxyMetadataModel]: Proxy and ProxyMetadata object.
        """
        response: dict[str, ProxyMetadataModel] = {}
        for proxy, metadata in proxies_util.fetch(
            proxies_util.url_map["proxies_metadata"]
        ).items():

            if not metadata.get("status", "") == "success":
                continue
            try:
                for filter_key, filter_value in filters.items():
                    metadata_value: typing.Any = metadata.get(filter_key)
                    if isinstance(metadata_value, (float, int)):
                        assert metadata_value <= filter_value
                    else:
                        assert metadata_value == filter_value
                response[proxy] = ProxyMetadataModel(**metadata)
            except:
                pass

        return response

    def get_confirmed_working_proxies(
        self, filters: dict[str, typing.Any] = {}
    ) -> list[str]:
        """List of tested working proxies. Filters are supported.

        Args:
            filters (dict[str, typing.Any]): Proxy `metadata key` and it's corresponding `value`. Defaults to `{}`.
             - Case type of filter is `int|float`, comparator will be `<=` else `==`.

        Returns:
            list[str]: Proxies.
        """
        response: list[str] = []
        for proxy, metadata in proxies_util.fetch(
            proxies_util.url_map["proxies_metadata"]
        ).items():
            try:
                for filter_key, filter_value in filters.items():
                    metadata_value: typing.Any = metadata.get(filter_key)
                    if isinstance(metadata_value, (float, int)):
                        assert metadata_value <= filter_value
                    else:
                        assert metadata_value == filter_value
                response.append(proxy)
            except:
                pass

        return response

    def get_proxies_generation_logs(self) -> str:
        "Last proxies generation logs"
        return proxies_util.session.get(
            proxies_util.url_map["proxies_generation_logs"],
            timeout=proxies_util.requests_timeout,
        ).text

    def update(self) -> "FreeProxies":
        """Create new class instance and update timestamp"""
        return FreeProxies()



import logging
import requests
import typing
from functools import wraps
from pyfreeproxies.models import ProxyMetadataModel

contents_url: str = (
    "https://raw.githubusercontent.com/Simatwa/free-proxies/master/files"
)

proxies_update_frequency_in_seconds: int = 30 * 60

requests_timeout: int = 10

session = requests.Session()

session.headers = {
    "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:124.0) Gecko/20100101 Firefox/124.0",
    "Accept": "*/*",
}


url_map = {
    "http_proxies": contents_url + "/http.json",
    "socks4_proxies": contents_url + "/socks4.json",
    "socks5_proxies": contents_url + "/socks5.json",
    "combined_proxies": contents_url + "/proxies.json",
    "random_proxies": contents_url + "/random.json",
    "proxies_metadata": contents_url + "/metadata.json",
    "last_proxies_updated_time": contents_url + "/timestamp.json",
    "proxies_generation_logs": contents_url + "/proxies.log",
}


def exception_handler(func):
    """Handle execptions accordingly"""

    @wraps(func)
    def decorator(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logging.error(e)

    return decorator


def fetch(*args, **kwargs) -> dict[str, typing.Any]:
    """fetch resource from internet"""
    kwargs["timeout"] = requests_timeout
    resp = session.get(*args, **kwargs)
    resp.raise_for_status()
    return resp.json()


def trace_ip(ip: str) -> ProxyMetadataModel:
    """Trace particular IP

    Args:
        ip (str): IP address.

    Returns:
        ProxyMetadataModel: Ip metadata
    """
    query = ip.replace("https://", "").replace("http://", "").split(":")[0]
    return ProxyMetadataModel(**fetch(f"http://ip-api.com/json/{query}"))


def trace_me() -> ProxyMetadataModel:
    """Get your ip metadata"""
    return ProxyMetadataModel(**fetch("http://ip-api.com/json"))



import unittest
from pyfreeproxies import FreeProxies
from pyfreeproxies.models import ProxyMetadataModel
from pyfreeproxies.utils import trace_ip


class TestFreeProxies(unittest.TestCase):

    def setUp(self):
        self.proxy = FreeProxies()

    def test_timestamp_check(self):
        "Timestamp check"
        self.assertIsInstance(self.proxy.proxies_update_available, bool)

    def test_http_proxies(self):
        "Http proxies fetch"
        self.assertIsInstance(self.proxy.get_http_proxies(), list)

    def test_socks4_proxies(self):
        """Socks4 proxies fetch"""
        self.assertIsInstance(self.proxy.get_socks4_proxies(), list)

    def test_socks5_proxies(self):
        """Scoks5 proxies fetch"""
        self.assertIsInstance(self.proxy.get_socks5_proxies(), list)

    def test_random_proxies(self):
        """Random proxies fetch"""
        self.assertIsInstance(self.proxy.get_random_proxies(), list)

    def test_combined_proxies(self):
        """Combined proxies fetch"""
        self.assertIsInstance(self.proxy.get_combined_proxies(), dict)

    def test_proxy_metadata(self):
        """Proxes' metadata fetch"""
        metadata = self.proxy.get_proxies_metadata()
        self.assertTrue(type(metadata) == dict)
        self.assertIsInstance(list(metadata.values())[0], ProxyMetadataModel)

    def test_confirmed_working_proxies(self):
        """Confirmed working proxies"""
        filter = {"country": "United States"}
        working_proxies = self.proxy.get_confirmed_working_proxies(filter)
        self.assertIsInstance(working_proxies, list)
        if working_proxies:
            ip_metadata = trace_ip(working_proxies[0])
            assert ip_metadata.country == filter["country"]

    def test_proxies_logs_availability(self):
        """Proxies generation logs availability"""
        self.assertIsInstance(self.proxy.get_proxies_generation_logs(), str)

    def test_update(self):
        """Updare FreeProxies"""
        new_FreeProxies = self.proxy.update()
        assert hasattr(new_FreeProxies, "update")


if __name__ == "__main__":
    unittest.main()



from pathlib import Path

from setuptools import setup

from setuptools import find_packages

DOCS_PATH = Path(__file__).parents[0] / "docs/README.md"
PATH = Path("README.md")
if not PATH.exists():
    with Path.open(DOCS_PATH, encoding="utf-8") as f1:
        with Path.open(PATH, "w+", encoding="utf-8") as f2:
            f2.write(f1.read())

setup(
    name="pyfreeproxies",
    version="0.0.2",
    license="GNUv3",
    author="Smartwa",
    maintainer="Smartwa",
    author_email="simatwacaleb@proton.me",
    description="Free to use http, socks4 and socks5 proxies",
    packages=find_packages("src"),
    package_dir={"": "src"},
    url="https://github.com/Simatwa/pyfreeproxies",
    project_urls={
        "Bug Report": "https://github.com/Simatwa/pyfreeproxies/issues/new",
        "Homepage": "https://github.com/Simatwa/pyfreeproxies",
        "Source Code": "https://github.com/Simatwa/pyfreeproxies",
        "Issue Tracker": "https://github.com/Simatwa/pyfreeproxies/issues",
        "Download": "https://github.com/Simatwa/pyfreeproxies/releases",
        "Documentation": "https://github.com/Simatwa/pyfreeproxies/blob/main/docs",
    },
    entry_points={
        "console_scripts": [
            "pytgpt = pytgpt.console:main",
        ],
    },
    install_requires=["requests>=2.31.0", "pydantic>=2.6.4"],
    python_requires=">=3.9",
    keywords=[
        "freeproxies",
        "proxies",
        "socks4",
        "socks5",
        "http",
    ],
    long_description=Path.open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Intended Audience :: Developers",
        "Topic :: Internet :: Proxy Servers",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
)
